@@ -4,7 +4,7 @@
  *
  * Copyright © 2007 Canonical Ltd
  * Written by Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2008-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2008-2015 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <unistd.h>
 
 #include <dpkg/i18n.h>
+#include <dpkg/c-ctype.h>
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
 #include <dpkg/pkg.h>
@@ -84,7 +85,7 @@ static struct trig_hooks trigh;
 static void
 trig_record_activation(struct pkginfo *pend, struct pkginfo *aw, const char *trig)
 {
-	if (pend->status < stat_triggersawaited)
+	if (pend->status < PKG_STAT_TRIGGERSAWAITED)
 		return; /* Not interested then. */
 
 	if (trig_note_pend(pend, trig))
@@ -93,10 +94,10 @@ trig_record_activation(struct pkginfo *pend, struct pkginfo *aw, const char *tri
 	if (trigh.enqueue_deferred)
 		trigh.enqueue_deferred(pend);
 
-	if (aw && pend->status > stat_configfiles)
+	if (aw && pend->status > PKG_STAT_CONFIGFILES)
 		if (trig_note_aw(pend, aw)) {
-			if (aw->status > stat_triggersawaited)
-				pkg_set_status(aw, stat_triggersawaited);
+			if (aw->status > PKG_STAT_TRIGGERSAWAITED)
+				pkg_set_status(aw, PKG_STAT_TRIGGERSAWAITED);
 			modstatdb_note_ifwrite(aw);
 		}
 }
@@ -116,11 +117,11 @@ trig_clear_awaiters(struct pkginfo *notpend)
 		if (!aw)
 			continue;
 		LIST_UNLINK_PART(aw->trigaw, ta, sameaw.);
-		if (!aw->trigaw.head && aw->status == stat_triggersawaited) {
+		if (!aw->trigaw.head && aw->status == PKG_STAT_TRIGGERSAWAITED) {
 			if (aw->trigpend_head)
-				pkg_set_status(aw, stat_triggerspending);
+				pkg_set_status(aw, PKG_STAT_TRIGGERSPENDING);
 			else
-				pkg_set_status(aw, stat_installed);
+				pkg_set_status(aw, PKG_STAT_INSTALLED);
 			modstatdb_note(aw);
 		}
 	}
@@ -163,7 +164,7 @@ static const struct trigkindinfo tki_explicit, tki_file, tki_unknown;
 static const struct trigkindinfo *dtki;
 
 /* As passed into activate_start. */
-static const char *trig_activating_name;
+static char *trig_activating_name;
 
 static const struct trigkindinfo *
 trig_classify_byname(const char *name)
@@ -190,16 +191,16 @@ invalid:
 
 /*
  * Calling sequence is:
- *  trig_activate_start(triggername)
- *  dtki->activate_awaiter(awaiting_package) } zero or more times
- *  dtki->activate_awaiter(0)                }  in any order
- *  dtki->activate_done(0)
+ *  trig_activate_start(triggername);
+ *  dtki->activate_awaiter(awaiting_package); } zero or more times
+ *  dtki->activate_awaiter(NULL);             }  in any order
+ *  dtki->activate_done();
  */
 static void
 trig_activate_start(const char *name)
 {
 	dtki = trig_classify_byname(name);
-	trig_activating_name = name;
+	trig_activating_name = nfstrsave(name);
 	dtki->activate_start();
 }
 
@@ -225,8 +226,8 @@ trk_unknown_interest_change(const char *trig, struct pkginfo *pkg,
                             struct pkgbin *pkgbin, int signum,
                             enum trig_options opts)
 {
-	ohshit(_("invalid or unknown syntax in trigger name `%.250s'"
-	         " (in trigger interests for package `%.250s')"),
+	ohshit(_("invalid or unknown syntax in trigger name '%.250s'"
+	         " (in trigger interests for package '%.250s')"),
 	       trig, pkgbin_name(pkg, pkgbin, pnaw_nonambig));
 }
 
@@ -266,7 +267,7 @@ trk_explicit_start(const char *trig)
 	trk_explicit_f = fopen(trk_explicit_fn.buf, "r");
 	if (!trk_explicit_f) {
 		if (errno != ENOENT)
-			ohshite(_("failed to open trigger interest list file `%.250s'"),
+			ohshite(_("failed to open trigger interest list file '%.250s'"),
 			        trk_explicit_fn.buf);
 	}
 }
@@ -281,7 +282,7 @@ static void
 trk_explicit_activate_start(void)
 {
 	trk_explicit_start(trig_activating_name);
-	trk_explicit_trig = nfstrsave(trig_activating_name);
+	trk_explicit_trig = trig_activating_name;
 }
 
 static void
@@ -294,7 +295,7 @@ trk_explicit_activate_awaiter(struct pkginfo *aw)
 		return;
 
 	if (fseek(trk_explicit_f, 0, SEEK_SET))
-		ohshite(_("failed to rewind trigger interest file `%.250s'"),
+		ohshite(_("failed to rewind trigger interest file '%.250s'"),
 		        trk_explicit_fn.buf);
 
 	while (trk_explicit_fgets(buf, sizeof(buf)) >= 0) {
@@ -306,11 +307,15 @@ trk_explicit_activate_awaiter(struct pkginfo *aw)
 			noawait = true;
 			*slash = '\0';
 		}
+		if (slash && strcmp("/await", slash) == 0) {
+			noawait = false;
+			*slash = '\0';
+		}
 
 		pend = pkg_spec_parse_pkg(buf, &err);
 		if (pend == NULL)
-			ohshit(_("trigger interest file `%.250s' syntax error; "
-			         "illegal package name `%.250s': %.250s"),
+			ohshit(_("trigger interest file '%.250s' syntax error; "
+			         "illegal package name '%.250s': %.250s"),
 			       trk_explicit_fn.buf, buf, err.str);
 
 		trig_record_activation(pend, noawait ? NULL : aw,
@@ -333,9 +338,9 @@ trk_explicit_interest_change(const char *trig,  struct pkginfo *pkg,
 
 	while (trk_explicit_f && trk_explicit_fgets(buf, sizeof(buf)) >= 0) {
 		const char *pkgname = pkgbin_name(pkg, pkgbin, pnaw_nonambig);
-		int len = strlen(pkgname);
+		size_t len = strlen(pkgname);
 
-		if (strncmp(buf, pkgname, len) == 0 &&
+		if (strncmp(buf, pkgname, len) == 0 && len < sizeof(buf) &&
 		    (buf[len] == '\0' || buf[len] == '/'))
 			continue;
 		fprintf(file->fp, "%s\n", buf);
@@ -344,7 +349,7 @@ trk_explicit_interest_change(const char *trig,  struct pkginfo *pkg,
 	if (signum > 0) {
 		fprintf(file->fp, "%s%s\n",
 		        pkgbin_name(pkg, pkgbin, pnaw_nonambig),
-		        (opts == trig_noawait) ? "/noawait" : "");
+		        (opts == TRIG_NOAWAIT) ? "/noawait" : "");
 		empty = false;
 	}
 
@@ -427,8 +432,8 @@ trk_file_interest_change(const char *trig, struct pkginfo *pkg,
 found:
 	tfi->options = opts;
 	if (signum > 1)
-		ohshit(_("duplicate file trigger interest for filename `%.250s' "
-		         "and package `%.250s'"), trig,
+		ohshit(_("duplicate file trigger interest for filename '%.250s' "
+		         "and package '%.250s'"), trig,
 		       pkgbin_name(pkg, pkgbin, pnaw_nonambig));
 	if (signum > 0)
 		return;
@@ -444,7 +449,7 @@ static void
 trig_file_interests_remove(void)
 {
 	if (unlink(triggersfilefile) && errno != ENOENT)
-		ohshite(_("cannot remove `%.250s'"), triggersfilefile);
+		ohshite(_("cannot remove '%.250s'"), triggersfilefile);
 }
 
 static void
@@ -459,7 +464,7 @@ trig_file_interests_update(void)
 	for (tfi = filetriggers.head; tfi; tfi = tfi->inoverall.next)
 		fprintf(file->fp, "%s %s%s\n", trigh.namenode_name(tfi->fnn),
 		        pkgbin_name(tfi->pkg, tfi->pkgbin, pnaw_nonambig),
-		        (tfi->options == trig_noawait) ? "/noawait" : "");
+		        (tfi->options == TRIG_NOAWAIT) ? "/noawait" : "");
 
 	atomic_file_sync(file);
 	atomic_file_close(file);
@@ -498,7 +503,7 @@ trig_file_interests_ensure(void)
 	if (!f) {
 		if (errno == ENOENT)
 			goto ok;
-		ohshite(_("unable to read file triggers file `%.250s'"),
+		ohshite(_("unable to read file triggers file '%.250s'"),
 		        triggersfilefile);
 	}
 
@@ -506,24 +511,28 @@ trig_file_interests_ensure(void)
 	while (fgets_checked(linebuf, sizeof(linebuf), f, triggersfilefile) >= 0) {
 		struct dpkg_error err;
 		char *slash;
-		enum trig_options trig_opts = trig_await;
+		enum trig_options trig_opts = TRIG_AWAIT;
 		space = strchr(linebuf, ' ');
 		if (!space || linebuf[0] != '/')
-			ohshit(_("syntax error in file triggers file `%.250s'"),
+			ohshit(_("syntax error in file triggers file '%.250s'"),
 			       triggersfilefile);
 		*space++ = '\0';
 
 		slash = strchr(space, '/');
 		if (slash && strcmp("/noawait", slash) == 0) {
-			trig_opts = trig_noawait;
+			trig_opts = TRIG_NOAWAIT;
+			*slash = '\0';
+		}
+		if (slash && strcmp("/await", slash) == 0) {
+			trig_opts = TRIG_AWAIT;
 			*slash = '\0';
 		}
 
 		pkg = pkg_spec_parse_pkg(space, &err);
 		if (pkg == NULL)
 			ohshit(_("file triggers record mentions illegal "
-			         "package name `%.250s' (for interest in file "
-			         "`%.250s'): %.250s"), space, linebuf, err.str);
+			         "package name '%.250s' (for interest in file "
+			         "'%.250s'): %.250s"), space, linebuf, err.str);
 		pkgbin = &pkg->installed;
 
 		trk_file_interest_change(linebuf, pkg, pkgbin, +2, trig_opts);
@@ -549,7 +558,7 @@ trig_file_activate(struct filenamenode *trig, struct pkginfo *aw)
 
 	for (tfi = *trigh.namenode_interested(trig); tfi;
 	     tfi = tfi->samefile_next)
-		trig_record_activation(tfi->pkg, (tfi->options == trig_noawait) ?
+		trig_record_activation(tfi->pkg, (tfi->options == TRIG_NOAWAIT) ?
 		                       NULL : aw, trigh.namenode_name(trig));
 }
 
@@ -592,7 +601,7 @@ static const char *trk_file_trig;
 static void
 trk_file_activate_start(void)
 {
-	trk_file_trig = nfstrsave(trig_activating_name);
+	trk_file_trig = trig_activating_name;
 }
 
 static void
@@ -647,7 +656,7 @@ trig_cicb_statuschange_activate(const char *trig, struct pkginfo *pkg,
 	struct pkginfo *aw = pkg;
 
 	trig_activate_start(trig);
-	dtki->activate_awaiter((opts == trig_noawait) ? NULL : aw);
+	dtki->activate_awaiter((opts == TRIG_NOAWAIT) ? NULL : aw);
 	dtki->activate_done();
 }
 
@@ -660,8 +669,8 @@ parse_ci_call(const char *file, const char *cmd, trig_parse_cicb *cb,
 
 	emsg = trig_name_is_illegal(trig);
 	if (emsg)
-		ohshit(_("triggers ci file `%.250s' contains illegal trigger "
-		         "syntax in trigger name `%.250s': %.250s"),
+		ohshit(_("triggers ci file '%.250s' contains illegal trigger "
+		         "syntax in trigger name '%.250s': %.250s"),
 		       file, trig, emsg);
 	if (cb)
 		cb(trig, pkg, pkgbin, opts);
@@ -680,35 +689,37 @@ trig_parse_ci(const char *file, trig_parse_cicb *interest,
 	if (!f) {
 		if (errno == ENOENT)
 			return; /* No file is just like an empty one. */
-		ohshite(_("unable to open triggers ci file `%.250s'"), file);
+		ohshite(_("unable to open triggers ci file '%.250s'"), file);
 	}
 	push_cleanup(cu_closestream, ~0, NULL, 0, 1, f);
 
 	while ((l = fgets_checked(linebuf, sizeof(linebuf), f, file)) >= 0) {
-		for (cmd = linebuf; cisspace(*cmd); cmd++);
+		for (cmd = linebuf; c_iswhite(*cmd); cmd++) ;
 		if (*cmd == '#')
 			continue;
-		for (eol = linebuf + l; eol > cmd && cisspace(eol[-1]); eol--);
+		for (eol = linebuf + l; eol > cmd && c_iswhite(eol[-1]); eol--) ;
 		if (eol == cmd)
 			continue;
 		*eol = '\0';
 
-		for (spc = cmd; *spc && !cisspace(*spc); spc++);
+		for (spc = cmd; *spc && !c_iswhite(*spc); spc++) ;
 		if (!*spc)
 			ohshit(_("triggers ci file contains unknown directive syntax"));
 		*spc++ = '\0';
-		while (cisspace(*spc))
+		while (c_iswhite(*spc))
 			spc++;
-		if (strcmp(cmd, "interest") == 0) {
-			parse_ci_call(file, cmd, interest, spc, pkg, pkgbin, trig_await);
+		if (strcmp(cmd, "interest") == 0 ||
+		    strcmp(cmd, "interest-await") == 0) {
+			parse_ci_call(file, cmd, interest, spc, pkg, pkgbin, TRIG_AWAIT);
 		} else if (strcmp(cmd, "interest-noawait") == 0) {
-			parse_ci_call(file, cmd, interest, spc, pkg, pkgbin, trig_noawait);
-		} else if (strcmp(cmd, "activate") == 0) {
-			parse_ci_call(file, cmd, activate, spc, pkg, pkgbin, trig_await);
+			parse_ci_call(file, cmd, interest, spc, pkg, pkgbin, TRIG_NOAWAIT);
+		} else if (strcmp(cmd, "activate") == 0 ||
+		           strcmp(cmd, "activate-await") == 0) {
+			parse_ci_call(file, cmd, activate, spc, pkg, pkgbin, TRIG_AWAIT);
 		} else if (strcmp(cmd, "activate-noawait") == 0) {
-			parse_ci_call(file, cmd, activate, spc, pkg, pkgbin, trig_noawait);
+			parse_ci_call(file, cmd, activate, spc, pkg, pkgbin, TRIG_NOAWAIT);
 		} else {
-			ohshit(_("triggers ci file contains unknown directive `%.250s'"),
+			ohshit(_("triggers ci file contains unknown directive '%.250s'"),
 			       cmd);
 		}
 	}
@@ -752,7 +763,7 @@ void
 trig_incorporate(enum modstatdb_rw cstatus)
 {
 	enum trigdef_update_status ur;
-	enum trigdef_updateflags tduf;
+	enum trigdef_update_flags tduf;
 
 	free(triggersdir);
 	triggersdir = dpkg_db_get_path(TRIGGERSDIR);
@@ -763,37 +774,37 @@ trig_incorporate(enum modstatdb_rw cstatus)
 	trigdef_set_methods(&tdm_incorp);
 	trig_file_interests_ensure();
 
-	tduf = tduf_nolockok;
+	tduf = TDUF_NO_LOCK_OK;
 	if (cstatus >= msdbrw_write) {
-		tduf |= tduf_write;
+		tduf |= TDUF_WRITE;
 		if (trigh.transitional_activate)
-			tduf |= tduf_writeifenoent;
+			tduf |= TDUF_WRITE_IF_ENOENT;
 	}
 
 	ur = trigdef_update_start(tduf);
-	if (ur == tdus_error_no_dir && cstatus >= msdbrw_write) {
+	if (ur == TDUS_ERROR_NO_DIR && cstatus >= msdbrw_write) {
 		if (mkdir(triggersdir, 0755)) {
 			if (errno != EEXIST)
 				ohshite(_("unable to create triggers state"
-				          " directory `%.250s'"), triggersdir);
+				          " directory '%.250s'"), triggersdir);
 		} else if (chown(triggersdir, 0, 0)) {
 			ohshite(_("unable to set ownership of triggers state"
-			          " directory `%.250s'"), triggersdir);
+			          " directory '%.250s'"), triggersdir);
 		}
 		ur = trigdef_update_start(tduf);
 	}
 	switch (ur) {
-	case tdus_error_empty_deferred:
+	case TDUS_ERROR_EMPTY_DEFERRED:
 		return;
-	case tdus_error_no_dir:
-	case tdus_error_no_deferred:
+	case TDUS_ERROR_NO_DIR:
+	case TDUS_ERROR_NO_DEFERRED:
 		if (!trigh.transitional_activate)
 			return;
 	/* Fall through. */
-	case tdus_no_deferred:
+	case TDUS_NO_DEFERRED:
 		trigh.transitional_activate(cstatus);
 		break;
-	case tdus_ok:
+	case TDUS_OK:
 		/* Read and incorporate triggers. */
 		trigdef_parse();
 		break;
