@@ -1,4 +1,5 @@
 # Copyright © 2008-2011 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2008-2015 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,14 +29,22 @@ packages. While it supports both the extraction and the creation
 of source packages, the only API that is officially supported
 is the one that supports the extraction of the source package.
 
-=head1 FUNCTIONS
-
 =cut
 
 use strict;
 use warnings;
 
 our $VERSION = '1.01';
+our @EXPORT_OK = qw(
+    get_default_diff_ignore_regex
+    set_default_diff_ignore_regex
+    get_default_tar_ignore_pattern
+);
+
+use Exporter qw(import);
+use POSIX qw(:errno_h :sys_wait_h);
+use Carp;
+use File::Basename;
 
 use Dpkg::Gettext;
 use Dpkg::ErrorHandling;
@@ -48,15 +57,6 @@ use Dpkg::Path qw(check_files_are_the_same find_command);
 use Dpkg::IPC;
 use Dpkg::Vendor qw(run_vendor_hook);
 
-use Carp;
-use POSIX qw(:errno_h :sys_wait_h);
-use File::Basename;
-
-use Exporter qw(import);
-our @EXPORT_OK = qw(get_default_diff_ignore_regex
-                    set_default_diff_ignore_regex
-                    get_default_tar_ignore_pattern);
-
 my $diff_ignore_default_regex = '
 # Ignore general backup files
 (?:^|/).*~$|
@@ -71,7 +71,7 @@ my $diff_ignore_default_regex = '
 # File or directory names that should be ignored
 (?:^|/)(?:CVS|RCS|\.deps|\{arch\}|\.arch-ids|\.svn|
 \.hg(?:tags|sigs)?|_darcs|\.git(?:attributes|modules)?|
-\.shelf|_MTN|\.be|\.bzr(?:\.backup|tags)?)(?:$|/.*$)
+\.mailmap|\.shelf|_MTN|\.be|\.bzr(?:\.backup|tags)?)(?:$|/.*$)
 ';
 # Take out comments and newlines
 $diff_ignore_default_regex =~ s/^#.*$//mg;
@@ -110,6 +110,7 @@ our @tar_ignore_default_pattern = qw(
 .hgignore
 .hgsigs
 .hgtags
+.mailmap
 .shelf
 .svn
 CVS
@@ -121,9 +122,11 @@ _darcs
 );
 ## use critic
 
+=head1 FUNCTIONS
+
 =over 4
 
-=item my $string = get_default_diff_ignore_regex()
+=item $string = get_default_diff_ignore_regex()
 
 Returns the default diff ignore regex.
 
@@ -140,12 +143,12 @@ Set a regex as the new default diff ignore regex.
 =cut
 
 sub set_default_diff_ignore_regex {
-    my ($regex) = @_;
+    my $regex = shift;
 
     $diff_ignore_default_regex = $regex;
 }
 
-=item my @array = get_default_tar_ignore_pattern()
+=item @array = get_default_tar_ignore_pattern()
 
 Returns the default tar ignore pattern, as an array.
 
@@ -155,6 +158,12 @@ sub get_default_tar_ignore_pattern {
     return @tar_ignore_default_pattern;
 }
 
+=back
+
+=head1 METHODS
+
+=over 4
+
 =item $p = Dpkg::Source::Package->new(filename => $dscfile, options => {})
 
 Creates a new object corresponding to the source package described
@@ -209,7 +218,7 @@ sub new {
 }
 
 sub init_options {
-    my ($self) = @_;
+    my $self = shift;
     # Use full ignore list by default
     # note: this function is not called by V1 packages
     $self->{options}{diff_ignore_regex} ||= $diff_ignore_default_regex;
@@ -224,13 +233,20 @@ sub init_options {
          'debian/source/local-patch-header';
     # Skip debianization while specific to some formats has an impact
     # on code common to all formats
-    $self->{options}{skip_debianization} ||= 0;
+    $self->{options}{skip_debianization} //= 0;
+
+    # Set default compressor for new formats.
+    $self->{options}{compression} //= 'xz';
+    $self->{options}{comp_level} //= compression_get_property($self->{options}{compression},
+                                                              'default_level');
+    $self->{options}{comp_ext} //= compression_get_property($self->{options}{compression},
+                                                            'file_ext');
 }
 
 sub initialize {
     my ($self, $filename) = @_;
     my ($fn, $dir) = fileparse($filename);
-    error(_g('%s is not the name of a file'), $filename) unless $fn;
+    error(g_('%s is not the name of a file'), $filename) unless $fn;
     $self->{basedir} = $dir || './';
     $self->{filename} = $fn;
 
@@ -242,7 +258,7 @@ sub initialize {
 
     foreach my $f (qw(Source Version Files)) {
         unless (defined($fields->{$f})) {
-            error(_g('missing critical source control field %s'), $f);
+            error(g_('missing critical source control field %s'), $f);
         }
     }
 
@@ -254,16 +270,15 @@ sub initialize {
 sub upgrade_object_type {
     my ($self, $update_format) = @_;
     $update_format //= 1;
-    $self->{fields}{'Format'} = '1.0'
-        unless exists $self->{fields}{'Format'};
+    $self->{fields}{'Format'} //= '1.0';
     my $format = $self->{fields}{'Format'};
 
     if ($format =~ /^([\d\.]+)(?:\s+\((.*)\))?$/) {
         my ($version, $variant, $major, $minor) = ($1, $2, $1, undef);
 
         if (defined $variant and $variant ne lc $variant) {
-            error(_g("source package format '%s' is not supported: %s"),
-                  $format, _g('format variant must be in lowercase'));
+            error(g_("source package format '%s' is not supported: %s"),
+                  $format, g_('format variant must be in lowercase'));
         }
 
         $major =~ s/\.[\d\.]+$//;
@@ -276,12 +291,12 @@ sub upgrade_object_type {
             $self->{fields}{'Format'} .= " ($variant)" if defined $variant;
         }
         if ($@) {
-            error(_g("source package format '%s' is not supported: %s"),
+            error(g_("source package format '%s' is not supported: %s"),
                   $format, $@);
         }
         bless $self, $module;
     } else {
-        error(_g("invalid Format field `%s'"), $format);
+        error(g_("invalid Format field '%s'"), $format);
     }
 }
 
@@ -292,7 +307,7 @@ Returns the filename of the DSC file.
 =cut
 
 sub get_filename {
-    my ($self) = @_;
+    my $self = shift;
     return $self->{basedir} . $self->{filename};
 }
 
@@ -304,7 +319,7 @@ usually do not have any path information.
 =cut
 
 sub get_files {
-    my ($self) = @_;
+    my $self = shift;
     return $self->{checksums}->get_files();
 }
 
@@ -317,7 +332,7 @@ discovered, it immediately errors out.
 =cut
 
 sub check_checksums {
-    my ($self) = @_;
+    my $self = shift;
     my $checksums = $self->{checksums};
     # add_from_file verify the checksums if they are already existing
     foreach my $file ($checksums->get_files()) {
@@ -329,7 +344,8 @@ sub get_basename {
     my ($self, $with_revision) = @_;
     my $f = $self->{fields};
     unless (exists $f->{'Source'} and exists $f->{'Version'}) {
-        error(_g('source and version are required to compute the source basename'));
+        error(g_('%s and %s fields are required to compute the source basename'),
+              'Source', 'Version');
     }
     my $v = Dpkg::Version->new($f->{'Version'});
     my $vs = $v->as_string(omit_epoch => 1, omit_revision => !$with_revision);
@@ -338,15 +354,14 @@ sub get_basename {
 
 sub find_original_tarballs {
     my ($self, %opts) = @_;
-    $opts{extension} = compression_get_file_extension_regex()
-        unless exists $opts{extension};
-    $opts{include_main} = 1 unless exists $opts{include_main};
-    $opts{include_supplementary} = 1 unless exists $opts{include_supplementary};
+    $opts{extension} //= compression_get_file_extension_regex();
+    $opts{include_main} //= 1;
+    $opts{include_supplementary} //= 1;
     my $basename = $self->get_basename();
     my @tar;
     foreach my $dir ('.', $self->{basedir}, $self->{options}{origtardir}) {
         next unless defined($dir) and -d $dir;
-        opendir(my $dir_dh, $dir) or syserr(_g('cannot opendir %s'), $dir);
+        opendir(my $dir_dh, $dir) or syserr(g_('cannot opendir %s'), $dir);
         push @tar, map { "$dir/$_" } grep {
 		($opts{include_main} and
 		 /^\Q$basename\E\.orig\.tar\.$opts{extension}$/) or
@@ -381,7 +396,7 @@ then any problem will result in a fatal error.
 =cut
 
 sub check_signature {
-    my ($self) = @_;
+    my $self = shift;
     my $dsc = $self->get_filename();
     my @exec;
 
@@ -395,7 +410,7 @@ sub check_signature {
         push @exec, 'gpg', '--no-default-keyring', '-q', '--verify';
     }
     if (scalar(@exec)) {
-        if (defined $ENV{HOME} and -r "$ENV{HOME}/.gnupg/trustedkeys.gpg") {
+        if (length $ENV{HOME} and -r "$ENV{HOME}/.gnupg/trustedkeys.gpg") {
             push @exec, '--keyring', "$ENV{HOME}/.gnupg/trustedkeys.gpg";
         }
         foreach my $vendor_keyring (run_vendor_hook('keyrings')) {
@@ -415,27 +430,27 @@ sub check_signature {
             if ($gpg_status == 1 or ($gpg_status &&
                 $self->{options}{require_valid_signature}))
             {
-                error(_g('failed to verify signature on %s'), $dsc);
+                error(g_('failed to verify signature on %s'), $dsc);
             } elsif ($gpg_status) {
-                warning(_g('failed to verify signature on %s'), $dsc);
+                warning(g_('failed to verify signature on %s'), $dsc);
             }
         } else {
             subprocerr("@exec");
         }
     } else {
         if ($self->{options}{require_valid_signature}) {
-            error(_g("could not verify signature on %s since gpg isn't installed"), $dsc);
+            error(g_("could not verify signature on %s since gpg isn't installed"), $dsc);
         } else {
-            warning(_g("could not verify signature on %s since gpg isn't installed"), $dsc);
+            warning(g_("could not verify signature on %s since gpg isn't installed"), $dsc);
         }
     }
 }
 
 sub parse_cmdline_options {
     my ($self, @opts) = @_;
-    foreach (@opts) {
-        if (not $self->parse_cmdline_option($_)) {
-            warning(_g('%s is not a valid option for %s'), $_, ref($self));
+    foreach my $option (@opts) {
+        if (not $self->parse_cmdline_option($option)) {
+            warning(g_('%s is not a valid option for %s'), $option, ref $self);
         }
     }
 }
@@ -452,11 +467,16 @@ that if $targetdir already exists, it will be erased.
 =cut
 
 sub extract {
-    my $self = shift;
-    my $newdirectory = $_[0];
+    my ($self, $newdirectory) = @_;
 
     my ($ok, $error) = version_check($self->{fields}{'Version'});
-    error($error) unless $ok;
+    if (not $ok) {
+        if ($self->{options}{ignore_bad_version}) {
+            warning($error);
+        } else {
+            error($error);
+        }
+    }
 
     # Copy orig tarballs
     if ($self->{options}{copy_orig_tarballs}) {
@@ -477,7 +497,7 @@ sub extract {
     }
 
     # Try extract
-    eval { $self->do_extract(@_) };
+    eval { $self->do_extract($newdirectory) };
     if ($@) {
         run_exit_handlers();
         die $@;
@@ -492,7 +512,7 @@ sub extract {
 	unless (-e $format_file) {
 	    mkdir($srcdir) unless -e $srcdir;
 	    open(my $format_fh, '>', $format_file)
-	        or syserr(_g('cannot write %s'), $format_file);
+	        or syserr(g_('cannot write %s'), $format_file);
 	    print { $format_fh } $self->{fields}{'Format'} . "\n";
 	    close($format_fh);
 	}
@@ -503,15 +523,15 @@ sub extract {
     my @s = lstat($rules);
     if (not scalar(@s)) {
         unless ($! == ENOENT) {
-            syserr(_g('cannot stat %s'), $rules);
+            syserr(g_('cannot stat %s'), $rules);
         }
-        warning(_g('%s does not exist'), $rules)
+        warning(g_('%s does not exist'), $rules)
             unless $self->{options}{skip_debianization};
     } elsif (-f _) {
         chmod($s[2] | 0111, $rules)
-            or syserr(_g('cannot make %s executable'), $rules);
+            or syserr(g_('cannot make %s executable'), $rules);
     } else {
-        warning(_g('%s is not a plain file'), $rules);
+        warning(g_('%s is not a plain file'), $rules);
     }
 }
 
@@ -571,7 +591,7 @@ sub commit {
 
 sub do_commit {
     my ($self, $dir) = @_;
-    info(_g("'%s' is not supported by the source format '%s'"),
+    info(g_("'%s' is not supported by the source format '%s'"),
          'dpkg-source --commit', $self->{fields}{'Format'});
 }
 
@@ -583,15 +603,15 @@ sub write_dsc {
 	$fields->{$f} = $opts{override}{$f};
     }
 
-    unless($opts{nocheck}) {
+    unless ($opts{nocheck}) {
         foreach my $f (qw(Source Version)) {
             unless (defined($fields->{$f})) {
-                error(_g('missing information for critical output field %s'), $f);
+                error(g_('missing information for critical output field %s'), $f);
             }
         }
         foreach my $f (qw(Maintainer Architecture Standards-Version)) {
             unless (defined($fields->{$f})) {
-                warning(_g('missing information for output field %s'), $f);
+                warning(g_('missing information for output field %s'), $f);
             }
         }
     }
@@ -601,11 +621,9 @@ sub write_dsc {
     }
 
     my $filename = $opts{filename};
-    unless (defined $filename) {
-        $filename = $self->get_basename(1) . '.dsc';
-    }
+    $filename //= $self->get_basename(1) . '.dsc';
     open(my $dsc_fh, '>', $filename)
-        or syserr(_g('cannot write %s'), $filename);
+        or syserr(g_('cannot write %s'), $filename);
     $fields->apply_substvars($opts{substvars});
     $fields->output($dsc_fh);
     close($dsc_fh);
@@ -615,13 +633,17 @@ sub write_dsc {
 
 =head1 CHANGES
 
-=head2 Version 1.01
+=head2 Version 1.01 (dpkg 1.17.2)
 
 New functions: get_default_diff_ignore_regex(), set_default_diff_ignore_regex(),
 get_default_tar_ignore_pattern()
 
 Deprecated variables: $diff_ignore_default_regexp, @tar_ignore_default_pattern
 
+=head2 Version 1.00 (dpkg 1.16.1)
+
+Mark the module as public.
+
 =head1 AUTHOR
 
 Raphaël Hertzog, E<lt>hertzog@debian.orgE<gt>
