@@ -4,7 +4,7 @@
  *
  * Copyright © 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
  * Copyright © 2000 Wichert Akkerman <wakkerma@debian.org>
- * Copyright © 2007-2013 Guillem Jover <guillem@debian.org>
+ * Copyright © 2007-2015 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -31,7 +31,6 @@
 
 #include <assert.h>
 #include <errno.h>
-#include <ctype.h>
 #include <string.h>
 #include <time.h>
 #include <fcntl.h>
@@ -57,12 +56,6 @@
 #include <dpkg/options.h>
 #include <dpkg/triglib.h>
 
-#ifdef WITH_SELINUX
-#include <selinux/selinux.h>
-#include <selinux/avc.h>
-#include <selinux/label.h>
-#endif
-
 #include "filesdb.h"
 #include "main.h"
 #include "archives.h"
@@ -157,7 +150,7 @@ filesavespackage(struct fileinlist *file,
     /* A Multi-Arch: same package can share files and their presence in a
      * third package of the same set is not a sign that we can get rid of
      * it. */
-    if (pkgtobesaved->installed.multiarch == multiarch_same &&
+    if (pkgtobesaved->installed.multiarch == PKG_MULTIARCH_SAME &&
         thirdpkg->set == pkgtobesaved->set)
       continue;
 
@@ -196,12 +189,12 @@ md5hash_prev_conffile(struct pkginfo *pkg, char *oldhash, const char *oldname,
       continue;
     /* The hash in the Conffiles is only meaningful if the package
      * configuration has been at least tried. */
-    if (otherpkg->status < stat_unpacked)
+    if (otherpkg->status < PKG_STAT_UNPACKED)
       continue;
     /* If we are reinstalling, even if the other package is only unpacked,
      * we can always make use of the Conffiles hash value from an initial
      * installation, if that happened at all. */
-    if (otherpkg->status == stat_unpacked &&
+    if (otherpkg->status == PKG_STAT_UNPACKED &&
         dpkg_version_compare(&otherpkg->installed.version,
                              &otherpkg->configversion) != 0)
       continue;
@@ -216,7 +209,7 @@ md5hash_prev_conffile(struct pkginfo *pkg, char *oldhash, const char *oldname,
       debug(dbg_conffdetail,
             "tarobject found shared conffile, from pkg %s (%s); hash=%s",
             pkg_name(otherpkg, pnaw_always),
-            statusinfos[otherpkg->status].name, oldhash);
+            pkg_status_name(otherpkg), oldhash);
       break;
     }
   }
@@ -232,7 +225,7 @@ md5hash_prev_conffile(struct pkginfo *pkg, char *oldhash, const char *oldname,
 }
 
 void cu_pathname(int argc, void **argv) {
-  ensure_pathname_nonexisting((char*)(argv[0]));
+  path_remove_tree((char*)(argv[0]));
 }
 
 int tarfileread(void *ud, char *buf, int len) {
@@ -264,7 +257,7 @@ tarobject_skip_entry(struct tarcontext *tc, struct tar_entry *ti)
 {
   /* We need to advance the tar file to the next object, so read the
    * file data and set it to oblivion. */
-  if (ti->type == tar_filetype_file) {
+  if (ti->type == TAR_FILETYPE_FILE) {
     struct dpkg_error err;
     char fnamebuf[256];
 
@@ -326,17 +319,22 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
   char *newhash;
 
   switch (te->type) {
-  case tar_filetype_file:
+  case TAR_FILETYPE_FILE:
     /* We create the file with mode 0 to make sure nobody can do anything with
      * it until we apply the proper mode, which might be a statoverride. */
     fd = open(path, O_CREAT | O_EXCL | O_WRONLY, 0);
     if (fd < 0)
-      ohshite(_("unable to create `%.255s' (while processing `%.255s')"),
+      ohshite(_("unable to create '%.255s' (while processing '%.255s')"),
               path, te->name);
     push_cleanup(cu_closefd, ehflag_bombout, NULL, 0, 1, &fd);
     debug(dbg_eachfiledetail, "tarobject file open size=%jd",
           (intmax_t)te->size);
 
+    /* We try to tell the filesystem how much disk space we are going to
+     * need to let it reduce fragmentation and possibly improve performance,
+     * as we do know the size beforehand. */
+    fd_allocate_size(fd, 0, te->size);
+
     newhash = nfmalloc(MD5HASHLEN + 1);
     if (fd_fd_copy_and_md5(tc->backendpipe, fd, newhash, te->size, &err) < 0)
       ohshit(_("cannot copy extracted data for '%.255s' to '%.255s': %s"),
@@ -355,9 +353,9 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
             namenode->statoverride->gid,
             namenode->statoverride->mode);
     if (fchown(fd, st->uid, st->gid))
-      ohshite(_("error setting ownership of `%.255s'"), te->name);
+      ohshite(_("error setting ownership of '%.255s'"), te->name);
     if (fchmod(fd, st->mode & ~S_IFMT))
-      ohshite(_("error setting permissions of `%.255s'"), te->name);
+      ohshite(_("error setting permissions of '%.255s'"), te->name);
 
     /* Postpone the fsync, to try to avoid massive I/O degradation. */
     if (!fc_unsafe_io)
@@ -365,27 +363,26 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
 
     pop_cleanup(ehflag_normaltidy); /* fd = open(path) */
     if (close(fd))
-      ohshite(_("error closing/writing `%.255s'"), te->name);
+      ohshite(_("error closing/writing '%.255s'"), te->name);
     break;
-  case tar_filetype_fifo:
+  case TAR_FILETYPE_FIFO:
     if (mkfifo(path, 0))
-      ohshite(_("error creating pipe `%.255s'"), te->name);
+      ohshite(_("error creating pipe '%.255s'"), te->name);
     debug(dbg_eachfiledetail, "tarobject fifo");
     break;
-  case tar_filetype_chardev:
+  case TAR_FILETYPE_CHARDEV:
     if (mknod(path, S_IFCHR, te->dev))
-      ohshite(_("error creating device `%.255s'"), te->name);
+      ohshite(_("error creating device '%.255s'"), te->name);
     debug(dbg_eachfiledetail, "tarobject chardev");
     break;
-  case tar_filetype_blockdev:
+  case TAR_FILETYPE_BLOCKDEV:
     if (mknod(path, S_IFBLK, te->dev))
-      ohshite(_("error creating device `%.255s'"), te->name);
+      ohshite(_("error creating device '%.255s'"), te->name);
     debug(dbg_eachfiledetail, "tarobject blockdev");
     break;
-  case tar_filetype_hardlink:
+  case TAR_FILETYPE_HARDLINK:
     varbuf_reset(&hardlinkfn);
     varbuf_add_str(&hardlinkfn, instdir);
-    varbuf_add_char(&hardlinkfn, '/');
     linknode = findnamenode(te->linkname, 0);
     varbuf_add_str(&hardlinkfn,
                    namenodetouse(linknode, tc->pkg, &tc->pkg->available)->name);
@@ -393,20 +390,20 @@ tarobject_extract(struct tarcontext *tc, struct tar_entry *te,
       varbuf_add_str(&hardlinkfn, DPKGNEWEXT);
     varbuf_end_str(&hardlinkfn);
     if (link(hardlinkfn.buf, path))
-      ohshite(_("error creating hard link `%.255s'"), te->name);
+      ohshite(_("error creating hard link '%.255s'"), te->name);
     namenode->newhash = linknode->newhash;
     debug(dbg_eachfiledetail, "tarobject hardlink hash=%s", namenode->newhash);
     break;
-  case tar_filetype_symlink:
+  case TAR_FILETYPE_SYMLINK:
     /* We've already checked for an existing directory. */
     if (symlink(te->linkname, path))
-      ohshite(_("error creating symbolic link `%.255s'"), te->name);
+      ohshite(_("error creating symbolic link '%.255s'"), te->name);
     debug(dbg_eachfiledetail, "tarobject symlink creating");
     break;
-  case tar_filetype_dir:
+  case TAR_FILETYPE_DIR:
     /* We've already checked for an existing directory. */
     if (mkdir(path, 0))
-      ohshite(_("error creating directory `%.255s'"), te->name);
+      ohshite(_("error creating directory '%.255s'"), te->name);
     debug(dbg_eachfiledetail, "tarobject directory creating");
     break;
   default:
@@ -418,7 +415,7 @@ static void
 tarobject_hash(struct tarcontext *tc, struct tar_entry *te,
                struct filenamenode *namenode)
 {
-  if (te->type == tar_filetype_file) {
+  if (te->type == TAR_FILETYPE_FILE) {
     struct dpkg_error err;
     char fnamebuf[256];
     char *newhash;
@@ -431,7 +428,7 @@ tarobject_hash(struct tarcontext *tc, struct tar_entry *te,
 
     namenode->newhash = newhash;
     debug(dbg_eachfiledetail, "tarobject file hash=%s", namenode->newhash);
-  } else if (te->type == tar_filetype_hardlink) {
+  } else if (te->type == TAR_FILETYPE_HARDLINK) {
     struct filenamenode *linknode;
 
     linknode = findnamenode(te->linkname, 0);
@@ -450,129 +447,38 @@ tarobject_set_mtime(struct tar_entry *te, const char *path)
   tv[1].tv_sec = te->mtime;
   tv[1].tv_usec = 0;
 
-  if (te->type == tar_filetype_symlink) {
+  if (te->type == TAR_FILETYPE_SYMLINK) {
 #ifdef HAVE_LUTIMES
     if (lutimes(path, tv) && errno != ENOSYS)
-      ohshite(_("error setting timestamps of `%.255s'"), path);
+      ohshite(_("error setting timestamps of '%.255s'"), path);
 #endif
   } else {
     if (utimes(path, tv))
-      ohshite(_("error setting timestamps of `%.255s'"), path);
+      ohshite(_("error setting timestamps of '%.255s'"), path);
   }
 }
 
 static void
 tarobject_set_perms(struct tar_entry *te, const char *path, struct file_stat *st)
 {
-  if (te->type == tar_filetype_file)
+  if (te->type == TAR_FILETYPE_FILE)
     return; /* Already handled using the file descriptor. */
 
-  if (te->type == tar_filetype_symlink) {
+  if (te->type == TAR_FILETYPE_SYMLINK) {
     if (lchown(path, st->uid, st->gid))
-      ohshite(_("error setting ownership of symlink `%.255s'"), path);
+      ohshite(_("error setting ownership of symlink '%.255s'"), path);
   } else {
     if (chown(path, st->uid, st->gid))
-      ohshite(_("error setting ownership of `%.255s'"), path);
+      ohshite(_("error setting ownership of '%.255s'"), path);
     if (chmod(path, st->mode & ~S_IFMT))
-      ohshite(_("error setting permissions of `%.255s'"), path);
+      ohshite(_("error setting permissions of '%.255s'"), path);
   }
 }
 
-#ifdef WITH_SELINUX
-static struct selabel_handle *dpkg_sehandle;
-
-static struct selabel_handle *
-dpkg_selabel_get_handle(void)
-{
-  return dpkg_sehandle;
-}
-#endif
-
-static void
-dpkg_selabel_load(void)
-{
-#ifdef WITH_SELINUX
-  static int selinux_enabled = -1;
-
-  if (selinux_enabled < 0) {
-    int rc;
-
-    /* Set selinux_enabled if it is not already set (singleton). */
-    selinux_enabled = (is_selinux_enabled() > 0);
-    if (!selinux_enabled)
-      return;
-
-    /* Open the SELinux status notification channel, with fallback enabled
-     * for older kernels. */
-    rc = selinux_status_open(1);
-    if (rc < 0)
-      ohshit(_("cannot open security status notification channel"));
-
-    /* XXX: We could use selinux_set_callback() to redirect the errors from
-     * the other SELinux calls, but that does not seem worth it right now. */
-  } else if (selinux_enabled && selinux_status_updated()) {
-    /* The SELinux policy got updated in the kernel, usually after upgrading
-     * the package shipping it, we need to reload. */
-    selabel_close(dpkg_sehandle);
-  } else {
-    /* SELinux is either disabled or it does not need a reload. */
-    return;
-  }
-
-  dpkg_sehandle = selabel_open(SELABEL_CTX_FILE, NULL, 0);
-  if (dpkg_sehandle == NULL)
-    ohshite(_("cannot get security labeling handle"));
-#endif
-}
-
-static void
-dpkg_selabel_close(void)
-{
-#ifdef WITH_SELINUX
-  if (dpkg_sehandle == NULL)
-    return;
-
-  selinux_status_close();
-  selabel_close(dpkg_sehandle);
-  dpkg_sehandle = NULL;
-#endif
-}
-
 static void
 tarobject_set_se_context(const char *matchpath, const char *path, mode_t mode)
 {
-#ifdef WITH_SELINUX
-  struct selabel_handle *sehandle;
-  security_context_t scontext = NULL;
-  int ret;
-
-  /* If there's no file type, just give up. */
-  if ((mode & S_IFMT) == 0)
-    return;
-
-  /* If SELinux is not enabled just do nothing. */
-  sehandle = dpkg_selabel_get_handle();
-  if (sehandle == NULL)
-    return;
-
-  /*
-   * We use the _raw function variants here so that no translation happens
-   * from computer to human readable forms, to avoid issues when mcstransd
-   * has disappeared during the unpack process.
-   */
-
-  /* Do nothing if we can't figure out what the context is, or if it has
-   * no context; in which case the default context shall be applied. */
-  ret = selabel_lookup_raw(sehandle, &scontext, matchpath, mode & S_IFMT);
-  if (ret == -1 || (ret == 0 && scontext == NULL))
-    return;
-
-  ret = lsetfilecon_raw(path, scontext);
-  if (ret < 0 && errno != ENOTSUP)
-    ohshite(_("cannot set security context for file object '%s'"), path);
-
-  freecon(scontext);
-#endif /* WITH_SELINUX */
+  dpkg_selabel_set_context(matchpath, path, mode);
 }
 
 static void
@@ -587,10 +493,10 @@ tarobject_matches(struct tarcontext *tc,
   debug(dbg_eachfiledetail, "tarobject matches on-disk object?");
 
   switch (te->type) {
-  case tar_filetype_dir:
+  case TAR_FILETYPE_DIR:
     /* Nothing to check for a new directory. */
     return;
-  case tar_filetype_symlink:
+  case TAR_FILETYPE_SYMLINK:
     /* Symlinks to existing dirs have already been dealt with, only
      * reamin real symlinks where we can compare the target. */
     if (!S_ISLNK(stab->st_mode))
@@ -598,7 +504,7 @@ tarobject_matches(struct tarcontext *tc,
     linkname = m_malloc(stab->st_size + 1);
     linksize = readlink(fn_old, linkname, stab->st_size + 1);
     if (linksize < 0)
-      ohshite(_("unable to read link `%.255s'"), fn_old);
+      ohshite(_("unable to read link '%.255s'"), fn_old);
     else if (linksize != stab->st_size)
       ohshit(_("symbolic link '%.250s' size has changed from %jd to %zd"),
              fn_old, stab->st_size, linksize);
@@ -610,21 +516,21 @@ tarobject_matches(struct tarcontext *tc,
       free(linkname);
     }
     break;
-  case tar_filetype_chardev:
+  case TAR_FILETYPE_CHARDEV:
     if (S_ISCHR(stab->st_mode) && stab->st_rdev == te->dev)
       return;
     break;
-  case tar_filetype_blockdev:
+  case TAR_FILETYPE_BLOCKDEV:
     if (S_ISBLK(stab->st_mode) && stab->st_rdev == te->dev)
       return;
     break;
-  case tar_filetype_fifo:
+  case TAR_FILETYPE_FIFO:
     if (S_ISFIFO(stab->st_mode))
       return;
     break;
-  case tar_filetype_hardlink:
+  case TAR_FILETYPE_HARDLINK:
     /* Fall through. */
-  case tar_filetype_file:
+  case TAR_FILETYPE_FILE:
     /* Only check metadata for non-conffiles. */
     if (!(namenode->flags & fnnf_new_conff) &&
         !(S_ISREG(stab->st_mode) && te->size == stab->st_size))
@@ -661,39 +567,6 @@ void setupfnamevbs(const char *filename) {
         fnamevb.buf, fnametmpvb.buf, fnamenewvb.buf);
 }
 
-/**
- * Securely remove a pathname.
- *
- * This is a secure version of remove(3) using secure_unlink() instead of
- * unlink(2).
- *
- * @retval  0 On success.
- * @retval -1 On failure, just like unlink(2) & rmdir(2).
- */
-int
-secure_remove(const char *filename)
-{
-  int r, e;
-
-  if (!rmdir(filename)) {
-    debug(dbg_eachfiledetail, "secure_remove '%s' rmdir OK", filename);
-    return 0;
-  }
-
-  if (errno != ENOTDIR) {
-    e= errno;
-    debug(dbg_eachfiledetail, "secure_remove '%s' rmdir %s", filename,
-          strerror(e));
-    errno= e; return -1;
-  }
-
-  r = secure_unlink(filename);
-  e = errno;
-  debug(dbg_eachfiledetail, "secure_remove '%s' unlink %s",
-        filename, r ? strerror(e) : "OK");
-  errno= e; return r;
-}
-
 struct fileinlist *addfiletolist(struct tarcontext *tc,
 				 struct filenamenode *namenode) {
   struct fileinlist *nifd;
@@ -727,7 +600,7 @@ linktosameexistingdir(const struct tar_entry *ti, const char *fname,
   statr= stat(fname, &oldstab);
   if (statr) {
     if (!(errno == ENOENT || errno == ELOOP || errno == ENOTDIR))
-      ohshite(_("failed to stat (dereference) existing symlink `%.250s'"),
+      ohshite(_("failed to stat (dereference) existing symlink '%.250s'"),
               fname);
     return false;
   }
@@ -750,7 +623,7 @@ linktosameexistingdir(const struct tar_entry *ti, const char *fname,
   if (statr) {
     if (!(errno == ENOENT || errno == ELOOP || errno == ENOTDIR))
       ohshite(_("failed to stat (dereference) proposed new symlink target"
-                " `%.250s' for symlink `%.250s'"), symlinkfn->buf, fname);
+                " '%.250s' for symlink '%.250s'"), symlinkfn->buf, fname);
     return false;
   }
   if (!S_ISDIR(newstab.st_mode))
@@ -776,13 +649,19 @@ tarobject(void *ctx, struct tar_entry *ti)
   int statr;
   ssize_t r;
   struct stat stab, stabtmp;
-  struct file_stat *st;
+  struct file_stat nodestat;
   struct fileinlist *nifd, **oldnifd;
   struct pkgset *divpkgset;
   struct pkginfo *otherpkg;
 
   ensureobstackinit();
 
+  tar_entry_update_from_system(ti);
+
+  /* Perform some sanity checks on the tar entry. */
+  if (strchr(ti->name, '\n'))
+    ohshit(_("newline not allowed in archive object name '%.255s'"), ti->name);
+
   /* Append to list of files.
    * The trailing ‘/’ put on the end of names in tarfiles has already
    * been stripped by tar_extractor(). */
@@ -807,25 +686,27 @@ tarobject(void *ctx, struct tar_entry *ti)
 
     if (divpkgset) {
       forcibleerr(fc_overwritediverted,
-                  _("trying to overwrite `%.250s', which is the "
-                    "diverted version of `%.250s' (package: %.100s)"),
+                  _("trying to overwrite '%.250s', which is the "
+                    "diverted version of '%.250s' (package: %.100s)"),
                   nifd->namenode->name, nifd->namenode->divert->camefrom->name,
                   divpkgset->name);
     } else {
       forcibleerr(fc_overwritediverted,
-                  _("trying to overwrite `%.250s', which is the "
-                    "diverted version of `%.250s'"),
+                  _("trying to overwrite '%.250s', which is the "
+                    "diverted version of '%.250s'"),
                   nifd->namenode->name, nifd->namenode->divert->camefrom->name);
     }
   }
 
-  if (nifd->namenode->statoverride)
-    st = nifd->namenode->statoverride;
-  else
-    st = &ti->stat;
+  if (nifd->namenode->statoverride) {
+    nodestat = *nifd->namenode->statoverride;
+    nodestat.mode |= ti->stat.mode & S_IFMT;
+  } else {
+    nodestat = ti->stat;
+  }
 
   usenode = namenodetouse(nifd->namenode, tc->pkg, &tc->pkg->available);
-  usename = usenode->name + 1; /* Skip the leading '/'. */
+  usename = usenode->name;
 
   trig_file_activate(usenode, tc->pkg);
 
@@ -843,7 +724,7 @@ tarobject(void *ctx, struct tar_entry *ti)
   if (statr) {
     /* The lstat failed. */
     if (errno != ENOENT && errno != ENOTDIR)
-      ohshite(_("unable to stat `%.255s' (which I was about to install)"),
+      ohshite(_("unable to stat '%.255s' (which I was about to install)"),
               ti->name);
     /* OK, so it doesn't exist.
      * However, it's possible that we were in the middle of some other
@@ -851,14 +732,15 @@ tarobject(void *ctx, struct tar_entry *ti)
      * So, we see if we have .dpkg-tmp, and if so we restore it. */
     if (rename(fnametmpvb.buf,fnamevb.buf)) {
       if (errno != ENOENT && errno != ENOTDIR)
-        ohshite(_("unable to clean up mess surrounding `%.255s' before "
+        ohshite(_("unable to clean up mess surrounding '%.255s' before "
                   "installing another version"), ti->name);
       debug(dbg_eachfiledetail,"tarobject nonexistent");
     } else {
       debug(dbg_eachfiledetail,"tarobject restored tmp to main");
       statr= lstat(fnamevb.buf,&stab);
-      if (statr) ohshite(_("unable to stat restored `%.255s' before installing"
-                           " another version"), ti->name);
+      if (statr)
+        ohshite(_("unable to stat restored '%.255s' before installing"
+                  " another version"), ti->name);
     }
   } else {
     debug(dbg_eachfiledetail,"tarobject already exists");
@@ -869,7 +751,7 @@ tarobject(void *ctx, struct tar_entry *ti)
    * a file overwriting conflict. */
   existingdir = false;
   switch (ti->type) {
-  case tar_filetype_symlink:
+  case TAR_FILETYPE_SYMLINK:
     /* If it's already an existing directory, do nothing. */
     if (!statr && S_ISDIR(stab.st_mode)) {
       debug(dbg_eachfiledetail, "tarobject symlink exists as directory");
@@ -879,21 +761,21 @@ tarobject(void *ctx, struct tar_entry *ti)
         existingdir = true;
     }
     break;
-  case tar_filetype_dir:
+  case TAR_FILETYPE_DIR:
     /* If it's already an existing directory, do nothing. */
     if (!stat(fnamevb.buf,&stabtmp) && S_ISDIR(stabtmp.st_mode)) {
       debug(dbg_eachfiledetail, "tarobject directory exists");
       existingdir = true;
     }
     break;
-  case tar_filetype_file:
-  case tar_filetype_chardev:
-  case tar_filetype_blockdev:
-  case tar_filetype_fifo:
-  case tar_filetype_hardlink:
+  case TAR_FILETYPE_FILE:
+  case TAR_FILETYPE_CHARDEV:
+  case TAR_FILETYPE_BLOCKDEV:
+  case TAR_FILETYPE_FIFO:
+  case TAR_FILETYPE_HARDLINK:
     break;
   default:
-    ohshit(_("archive contained object `%.255s' of unknown type 0x%x"),
+    ohshit(_("archive contained object '%.255s' of unknown type 0x%x"),
            ti->name, ti->type);
   }
 
@@ -913,8 +795,8 @@ tarobject(void *ctx, struct tar_entry *ti)
        * is allowed when they are not getting in sync, otherwise the
        * file content must match the installed file. */
       if (otherpkg->set == tc->pkg->set &&
-          otherpkg->installed.multiarch == multiarch_same &&
-          tc->pkg->available.multiarch == multiarch_same) {
+          otherpkg->installed.multiarch == PKG_MULTIARCH_SAME &&
+          tc->pkg->available.multiarch == PKG_MULTIARCH_SAME) {
         if (statr == 0 && tc->pkgset_getting_in_sync)
           refcounting = true;
         debug(dbg_eachfiledetail, "tarobject ... shared with %s %s (syncing=%d)",
@@ -939,7 +821,7 @@ tarobject(void *ctx, struct tar_entry *ti)
        * not exist assume it's also a directory and skip further checks.
        * XXX: Ideally with more information about the installed files we
        * could perform more clever checks. */
-      if (statr != 0 && ti->type == tar_filetype_dir) {
+      if (statr != 0 && ti->type == TAR_FILETYPE_DIR) {
         debug(dbg_eachfile, "tarobject ... assuming shared directory");
         continue;
       }
@@ -955,11 +837,11 @@ tarobject(void *ctx, struct tar_entry *ti)
       /* Is the package with the conflicting file in the “config files only”
        * state? If so it must be a config file and we can silenty take it
        * over. */
-      if (otherpkg->status == stat_configfiles)
+      if (otherpkg->status == PKG_STAT_CONFIGFILES)
         continue;
 
       /* Perhaps we're removing a conflicting package? */
-      if (otherpkg->clientdata->istobe == itb_remove)
+      if (otherpkg->clientdata->istobe == PKG_ISTOBE_REMOVE)
         continue;
 
       /* Is the file an obsolete conffile in the other package
@@ -1064,8 +946,8 @@ tarobject(void *ctx, struct tar_entry *ti)
   } else {
     /* Now, at this stage we want to make sure neither of .dpkg-new and
      * .dpkg-tmp are hanging around. */
-    ensure_pathname_nonexisting(fnamenewvb.buf);
-    ensure_pathname_nonexisting(fnametmpvb.buf);
+    path_remove_tree(fnamenewvb.buf);
+    path_remove_tree(fnametmpvb.buf);
 
     /* Now we start to do things that we need to be able to undo
      * if something goes wrong. Watch out for the CLEANUP comments to
@@ -1078,7 +960,7 @@ tarobject(void *ctx, struct tar_entry *ti)
      */
 
     /* Extract whatever it is as .dpkg-new ... */
-    tarobject_extract(tc, ti, fnamenewvb.buf, st, nifd->namenode);
+    tarobject_extract(tc, ti, fnamenewvb.buf, &nodestat, nifd->namenode);
   }
 
   /* For shared files, check now if the object matches. */
@@ -1090,9 +972,9 @@ tarobject(void *ctx, struct tar_entry *ti)
   if (refcounting && !fc_overwrite)
     return 0;
 
-  tarobject_set_perms(ti, fnamenewvb.buf, st);
+  tarobject_set_perms(ti, fnamenewvb.buf, &nodestat);
   tarobject_set_mtime(ti, fnamenewvb.buf);
-  tarobject_set_se_context(fnamevb.buf, fnamenewvb.buf, st->mode);
+  tarobject_set_se_context(fnamevb.buf, fnamenewvb.buf, nodestat.mode);
 
   /*
    * CLEANUP: Now we have extracted the new object in .dpkg-new (or,
@@ -1117,12 +999,12 @@ tarobject(void *ctx, struct tar_entry *ti)
     /* Don't try to back it up if it didn't exist. */
     debug(dbg_eachfiledetail,"tarobject new - no backup");
   } else {
-    if (ti->type == tar_filetype_dir || S_ISDIR(stab.st_mode)) {
+    if (ti->type == TAR_FILETYPE_DIR || S_ISDIR(stab.st_mode)) {
       /* One of the two is a directory - can't do atomic install. */
       debug(dbg_eachfiledetail,"tarobject directory, nonatomic");
       nifd->namenode->flags |= fnnf_no_atomic_overwrite;
       if (rename(fnamevb.buf,fnametmpvb.buf))
-        ohshite(_("unable to move aside `%.255s' to install new version"),
+        ohshite(_("unable to move aside '%.255s' to install new version"),
                 ti->name);
     } else if (S_ISLNK(stab.st_mode)) {
       /* We can't make a symlink with two hardlinks, so we'll have to
@@ -1132,21 +1014,21 @@ tarobject(void *ctx, struct tar_entry *ti)
       varbuf_grow(&symlinkfn, stab.st_size + 1);
       r = readlink(fnamevb.buf, symlinkfn.buf, symlinkfn.size);
       if (r < 0)
-        ohshite(_("unable to read link `%.255s'"), ti->name);
+        ohshite(_("unable to read link '%.255s'"), ti->name);
       else if (r != stab.st_size)
         ohshit(_("symbolic link '%.250s' size has changed from %jd to %zd"),
                fnamevb.buf, stab.st_size, r);
       varbuf_trunc(&symlinkfn, r);
       varbuf_end_str(&symlinkfn);
       if (symlink(symlinkfn.buf,fnametmpvb.buf))
-        ohshite(_("unable to make backup symlink for `%.255s'"), ti->name);
+        ohshite(_("unable to make backup symlink for '%.255s'"), ti->name);
       if (lchown(fnametmpvb.buf,stab.st_uid,stab.st_gid))
-        ohshite(_("unable to chown backup symlink for `%.255s'"), ti->name);
+        ohshite(_("unable to chown backup symlink for '%.255s'"), ti->name);
       tarobject_set_se_context(fnamevb.buf, fnametmpvb.buf, stab.st_mode);
     } else {
       debug(dbg_eachfiledetail, "tarobject nondirectory, 'link' backup");
       if (link(fnamevb.buf,fnametmpvb.buf))
-        ohshite(_("unable to make backup link of `%.255s' before installing new version"),
+        ohshite(_("unable to make backup link of '%.255s' before installing new version"),
                 ti->name);
     }
   }
@@ -1156,14 +1038,14 @@ tarobject(void *ctx, struct tar_entry *ti)
    * in .dpkg-new.
    */
 
-  if (ti->type == tar_filetype_file || ti->type == tar_filetype_hardlink ||
-      ti->type == tar_filetype_symlink) {
+  if (ti->type == TAR_FILETYPE_FILE || ti->type == TAR_FILETYPE_HARDLINK ||
+      ti->type == TAR_FILETYPE_SYMLINK) {
     nifd->namenode->flags |= fnnf_deferred_rename;
 
     debug(dbg_eachfiledetail, "tarobject done and installation deferred");
   } else {
     if (rename(fnamenewvb.buf, fnamevb.buf))
-      ohshite(_("unable to install new version of `%.255s'"), ti->name);
+      ohshite(_("unable to install new version of '%.255s'"), ti->name);
 
     /*
      * CLEANUP: Now the new file is in the destination file, and the
@@ -1189,16 +1071,14 @@ tar_writeback_barrier(struct fileinlist *files, struct pkginfo *pkg)
 
   for (cfile = files; cfile; cfile = cfile->next) {
     struct filenamenode *usenode;
-    const char *usename;
     int fd;
 
     if (!(cfile->namenode->flags & fnnf_deferred_fsync))
       continue;
 
     usenode = namenodetouse(cfile->namenode, pkg, &pkg->available);
-    usename = usenode->name + 1; /* Skip the leading '/'. */
 
-    setupfnamevbs(usename);
+    setupfnamevbs(usenode->name);
 
     fd = open(fnamenewvb.buf, O_WRONLY);
     if (fd < 0)
@@ -1208,7 +1088,7 @@ tar_writeback_barrier(struct fileinlist *files, struct pkginfo *pkg)
      * anyway. */
     sync_file_range(fd, 0, 0, SYNC_FILE_RANGE_WAIT_BEFORE);
     if (close(fd))
-      ohshite(_("error closing/writing `%.255s'"), fnamenewvb.buf);
+      ohshite(_("error closing/writing '%.255s'"), fnamenewvb.buf);
   }
 }
 #else
@@ -1223,7 +1103,6 @@ tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
 {
   struct fileinlist *cfile;
   struct filenamenode *usenode;
-  const char *usename;
 
   tar_writeback_barrier(files, pkg);
 
@@ -1234,9 +1113,8 @@ tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
       continue;
 
     usenode = namenodetouse(cfile->namenode, pkg, &pkg->available);
-    usename = usenode->name + 1; /* Skip the leading '/'. */
 
-    setupfnamevbs(usename);
+    setupfnamevbs(usenode->name);
 
     if (cfile->namenode->flags & fnnf_deferred_fsync) {
       int fd;
@@ -1249,7 +1127,7 @@ tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
       if (fsync(fd))
         ohshite(_("unable to sync file '%.255s'"), fnamenewvb.buf);
       if (close(fd))
-        ohshite(_("error closing/writing `%.255s'"), fnamenewvb.buf);
+        ohshite(_("error closing/writing '%.255s'"), fnamenewvb.buf);
 
       cfile->namenode->flags &= ~fnnf_deferred_fsync;
     }
@@ -1257,7 +1135,7 @@ tar_deferred_extract(struct fileinlist *files, struct pkginfo *pkg)
     debug(dbg_eachfiledetail, "deferred extract needs rename");
 
     if (rename(fnamenewvb.buf, fnamevb.buf))
-      ohshite(_("unable to install new version of `%.255s'"),
+      ohshite(_("unable to install new version of '%.255s'"),
               cfile->namenode->name);
 
     cfile->namenode->flags &= ~fnnf_deferred_rename;
@@ -1282,7 +1160,7 @@ enqueue_deconfigure(struct pkginfo *pkg, struct pkginfo *pkg_removal)
   struct pkg_deconf_list *newdeconf;
 
   ensure_package_clientdata(pkg);
-  pkg->clientdata->istobe = itb_deconfigure;
+  pkg->clientdata->istobe = PKG_ISTOBE_DECONFIGURE;
   newdeconf = m_malloc(sizeof(struct pkg_deconf_list));
   newdeconf->next = deconfigure;
   newdeconf->pkg = pkg;
@@ -1375,7 +1253,7 @@ void check_breaks(struct dependency *dep, struct pkginfo *pkg,
     char action[512];
 
     ensure_package_clientdata(fixbydeconf);
-    assert(fixbydeconf->clientdata->istobe == itb_normal);
+    assert(fixbydeconf->clientdata->istobe == PKG_ISTOBE_NORMAL);
 
     sprintf(action, _("installation of %.250s"),
             pkgbin_name(pkg, &pkg->available, pnaw_nonambig));
@@ -1428,22 +1306,24 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
   }
   if (fixbyrm) {
     ensure_package_clientdata(fixbyrm);
-    if (fixbyrm->clientdata->istobe == itb_installnew) {
+    if (fixbyrm->clientdata->istobe == PKG_ISTOBE_INSTALLNEW) {
       fixbyrm= dep->up;
       ensure_package_clientdata(fixbyrm);
     }
     if (((pkg->available.essential && fixbyrm->installed.essential) ||
-         (((fixbyrm->want != want_install && fixbyrm->want != want_hold) ||
+         (((fixbyrm->want != PKG_WANT_INSTALL &&
+            fixbyrm->want != PKG_WANT_HOLD) ||
            does_replace(pkg, &pkg->available, fixbyrm, &fixbyrm->installed)) &&
           (!fixbyrm->installed.essential || fc_removeessential)))) {
-      assert(fixbyrm->clientdata->istobe == itb_normal || fixbyrm->clientdata->istobe == itb_deconfigure);
-      fixbyrm->clientdata->istobe= itb_remove;
+      assert(fixbyrm->clientdata->istobe == PKG_ISTOBE_NORMAL ||
+             fixbyrm->clientdata->istobe == PKG_ISTOBE_DECONFIGURE);
+      fixbyrm->clientdata->istobe = PKG_ISTOBE_REMOVE;
       notice(_("considering removing %s in favour of %s ..."),
              pkg_name(fixbyrm, pnaw_nonambig),
              pkgbin_name(pkg, &pkg->available, pnaw_nonambig));
-      if (!(fixbyrm->status == stat_installed ||
-            fixbyrm->status == stat_triggerspending ||
-            fixbyrm->status == stat_triggersawaited)) {
+      if (!(fixbyrm->status == PKG_STAT_INSTALLED ||
+            fixbyrm->status == PKG_STAT_TRIGGERSPENDING ||
+            fixbyrm->status == PKG_STAT_TRIGGERSAWAITED)) {
         notice(_("%s is not properly installed; ignoring any dependencies on it"),
                pkg_name(fixbyrm, pnaw_nonambig));
         pdep = NULL;
@@ -1486,7 +1366,7 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
       if (!pdep && skip_due_to_hold(fixbyrm)) {
         pdep= &flagdeppossi;
       }
-      if (!pdep && (fixbyrm->eflag & eflag_reinstreq)) {
+      if (!pdep && (fixbyrm->eflag & PKG_EFLAG_REINSTREQ)) {
         if (fc_removereinstreq) {
           notice(_("package %s requires reinstallation, but will "
                    "remove anyway as you requested"),
@@ -1507,7 +1387,7 @@ void check_conflict(struct dependency *dep, struct pkginfo *pkg,
         return;
       }
       /* Put it back. */
-      fixbyrm->clientdata->istobe = itb_normal;
+      fixbyrm->clientdata->istobe = PKG_ISTOBE_NORMAL;
     }
   }
   varbuf_end_str(&conflictwhy);
@@ -1526,7 +1406,7 @@ void cu_cidir(int argc, void **argv) {
   char *cidir= (char*)argv[0];
   char *cidirrest= (char*)argv[1];
   cidirrest[-1] = '\0';
-  ensure_pathname_nonexisting(cidir);
+  path_remove_tree(cidir);
 }
 
 void cu_fileslist(int argc, void **argv) {
@@ -1539,14 +1419,20 @@ archivefiles(const char *const *argv)
   const char *volatile thisarg;
   const char *const *volatile argp;
   jmp_buf ejbuf;
+  enum modstatdb_rw msdbflags;
 
   trigproc_install_hooks();
 
-  modstatdb_open(f_noact ?                          msdbrw_readonly :
-                 (cipaction->arg_int == act_avail ? msdbrw_readonly :
-                  fc_nonroot ?                      msdbrw_write :
-                                                    msdbrw_needsuperuser) |
-                 msdbrw_available_write);
+  if (f_noact)
+    msdbflags = msdbrw_readonly;
+  else if (cipaction->arg_int == act_avail)
+    msdbflags = msdbrw_readonly | msdbrw_available_write;
+  else if (fc_nonroot)
+    msdbflags = msdbrw_write;
+  else
+    msdbflags = msdbrw_needsuperuser;
+
+  modstatdb_open(msdbflags);
 
   checkpath();
   pkg_infodb_upgrade();
@@ -1554,7 +1440,7 @@ archivefiles(const char *const *argv)
   log_message("startup archives %s", cipaction->olong);
 
   if (f_recursive) {
-    int pi[2], nfiles, c, i, r;
+    int pi[2], nfiles, c, i, rc;
     pid_t pid;
     FILE *pf;
     static struct varbuf findoutput;
@@ -1601,9 +1487,9 @@ archivefiles(const char *const *argv)
     }
     if (ferror(pf)) ohshite(_("error reading find's pipe"));
     if (fclose(pf)) ohshite(_("error closing find's pipe"));
-    r = subproc_wait_check(pid, "find", PROCNOERR);
-    if (r != 0)
-      ohshit(_("find for --recursive returned unhandled error %i"),r);
+    rc = subproc_reap(pid, "find", SUBPROC_RETERROR);
+    if (rc != 0)
+      ohshit(_("find for --recursive returned unhandled error %i"), rc);
 
     if (!nfiles)
       ohshit(_("searched, but found no packages (files matching *.deb)"));
@@ -1631,15 +1517,12 @@ archivefiles(const char *const *argv)
   varbuf_reset(&fnamenewvb);
 
   varbuf_add_str(&fnamevb, instdir);
-  varbuf_add_char(&fnamevb, '/');
   varbuf_add_str(&fnametmpvb, instdir);
-  varbuf_add_char(&fnametmpvb, '/');
   varbuf_add_str(&fnamenewvb, instdir);
-  varbuf_add_char(&fnamenewvb, '/');
   fnameidlu= fnamevb.used;
 
   ensure_diversions();
-  ensure_statoverrides();
+  ensure_statoverrides(STATDB_PARSE_NORMAL);
 
   while ((thisarg = *argp++) != NULL) {
     if (setjmp(ejbuf)) {
@@ -1694,9 +1577,9 @@ archivefiles(const char *const *argv)
 bool
 wanttoinstall(struct pkginfo *pkg)
 {
-  int r;
+  int rc;
 
-  if (pkg->want != want_install && pkg->want != want_hold) {
+  if (pkg->want != PKG_WANT_INSTALL && pkg->want != PKG_WANT_HOLD) {
     if (f_alsoselect) {
       printf(_("Selecting previously unselected package %s.\n"),
              pkgbin_name(pkg, &pkg->available, pnaw_nonambig));
@@ -1708,15 +1591,15 @@ wanttoinstall(struct pkginfo *pkg)
     }
   }
 
-  if (pkg->eflag & eflag_reinstreq)
+  if (pkg->eflag & PKG_EFLAG_REINSTREQ)
     return true;
-  if (pkg->status < stat_unpacked)
+  if (pkg->status < PKG_STAT_UNPACKED)
     return true;
 
-  r = dpkg_version_compare(&pkg->available.version, &pkg->installed.version);
-  if (r > 0) {
+  rc = dpkg_version_compare(&pkg->available.version, &pkg->installed.version);
+  if (rc > 0) {
     return true;
-  } else if (r == 0) {
+  } else if (rc == 0) {
     /* Same version fully installed. */
     if (f_skipsame) {
       notice(_("version %.250s of %.250s already installed, skipping"),
@@ -1743,14 +1626,18 @@ wanttoinstall(struct pkginfo *pkg)
   }
 }
 
-struct fileinlist *newconff_append(struct fileinlist ***newconffileslastp_io,
-				   struct filenamenode *namenode) {
-  struct fileinlist *newconff;
+struct fileinlist *
+filenamenode_queue_push(struct filenamenode_queue *queue,
+                        struct filenamenode *namenode)
+{
+  struct fileinlist *node;
 
-  newconff= m_malloc(sizeof(struct fileinlist));
-  newconff->next = NULL;
-  newconff->namenode= namenode;
-  **newconffileslastp_io= newconff;
-  *newconffileslastp_io= &newconff->next;
-  return newconff;
+  node = m_malloc(sizeof(*node));
+  node->next = NULL;
+  node->namenode = namenode;
+
+  *queue->tail = node;
+  queue->tail = &node->next;
+
+  return node;
 }
