@@ -3,7 +3,7 @@
  *
  * Copyright © 1995 Ian Jackson
  * Copyright © 2000, 2001 Wichert Akkerman
- * Copyright © 2006-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006-2014 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -67,7 +67,7 @@ static void
 printversion(const struct cmdinfo *cip, const char *value)
 {
 	printf(_("Debian %s version %s.\n"), dpkg_get_progname(),
-	       DPKG_VERSION_ARCH);
+	       PACKAGE_RELEASE);
 
 	printf(_(
 "This is free software; see the GNU General Public License version 2 or\n"
@@ -121,9 +121,9 @@ usage(const struct cmdinfo *cip, const char *value)
 struct file {
 	const char *name;
 	enum {
-		file_stat_invalid,
-		file_stat_valid,
-		file_stat_nofile,
+		FILE_STAT_INVALID,
+		FILE_STAT_VALID,
+		FILE_STAT_NOFILE,
 	} stat_state;
 	struct stat stat;
 };
@@ -132,7 +132,7 @@ static void
 file_init(struct file *f, const char *filename)
 {
 	f->name = filename;
-	f->stat_state = file_stat_invalid;
+	f->stat_state = FILE_STAT_INVALID;
 }
 
 static void
@@ -140,7 +140,7 @@ file_stat(struct file *f)
 {
 	int ret;
 
-	if (f->stat_state != file_stat_invalid)
+	if (f->stat_state != FILE_STAT_INVALID)
 		return;
 
 	ret = lstat(f->name, &f->stat);
@@ -148,9 +148,9 @@ file_stat(struct file *f)
 		ohshite(_("cannot stat file '%s'"), f->name);
 
 	if (ret == 0)
-		f->stat_state = file_stat_valid;
+		f->stat_state = FILE_STAT_VALID;
 	else
-		f->stat_state = file_stat_nofile;
+		f->stat_state = FILE_STAT_NOFILE;
 }
 
 static void
@@ -177,7 +177,7 @@ check_rename(struct file *src, struct file *dst)
 
 	/* If the source file is not present and we are not going to do
 	 * the rename anyway there's no point in checking any further. */
-	if (src->stat_state == file_stat_nofile)
+	if (src->stat_state == FILE_STAT_NOFILE)
 		return false;
 
 	file_stat(dst);
@@ -194,12 +194,12 @@ check_rename(struct file *src, struct file *dst)
 	check_writable_dir(src);
 	check_writable_dir(dst);
 
-	if (src->stat_state == file_stat_valid &&
-	    dst->stat_state == file_stat_valid &&
+	if (src->stat_state == FILE_STAT_VALID &&
+	    dst->stat_state == FILE_STAT_VALID &&
 	    !(src->stat.st_dev == dst->stat.st_dev &&
 	      src->stat.st_ino == dst->stat.st_ino))
-		ohshit(_("rename involves overwriting `%s' with\n"
-		         "  different file `%s', not allowed"),
+		ohshit(_("rename involves overwriting '%s' with\n"
+		         "  different file '%s', not allowed"),
 		        dst->name, src->name);
 
 	return true;
@@ -246,10 +246,10 @@ file_copy(const char *src, const char *dst)
 static void
 file_rename(struct file *src, struct file *dst)
 {
-	if (src->stat_state == file_stat_nofile)
+	if (src->stat_state == FILE_STAT_NOFILE)
 		return;
 
-	if (dst->stat_state == file_stat_valid) {
+	if (dst->stat_state == FILE_STAT_VALID) {
 		if (unlink(src->name))
 			ohshite(_("rename: remove duplicate old link '%s'"),
 			        src->name);
@@ -361,7 +361,7 @@ divertdb_write(void)
 
 	dbname = dpkg_db_get_path(DIVERSIONSFILE);
 
-	file = atomic_file_new(dbname, aff_backup);
+	file = atomic_file_new(dbname, ATOMIC_FILE_BACKUP);
 	atomic_file_open(file);
 
 	iter = files_db_iter_new();
@@ -431,7 +431,7 @@ diversion_add(const char *const *argv)
 	file_init(&file_from, filename);
 	file_stat(&file_from);
 
-	if (file_from.stat_state == file_stat_valid &&
+	if (file_from.stat_state == FILE_STAT_VALID &&
 	    S_ISDIR(file_from.stat.st_mode))
 		badusage(_("cannot divert directories"));
 
@@ -471,7 +471,7 @@ diversion_add(const char *const *argv)
 			exit(0);
 		}
 
-		ohshit(_("`%s' clashes with `%s'"),
+		ohshit(_("'%s' clashes with '%s'"),
 		       diversion_current(filename),
 		       fnn_from->divert ?
 		       diversion_describe(fnn_from->divert) :
@@ -528,7 +528,7 @@ diversion_is_shared(struct pkgset *set, struct filenamenode *namenode)
 
 	archname = getenv("DPKG_MAINTSCRIPT_ARCH");
 	arch = dpkg_arch_find(archname);
-	if (arch->type == arch_none || arch->type == arch_empty)
+	if (arch->type == DPKG_ARCH_NONE || arch->type == DPKG_ARCH_EMPTY)
 		return false;
 
 	for (pkg = &set->pkg; pkg; pkg = pkg->arch_next)
@@ -581,15 +581,15 @@ diversion_remove(const char *const *argv)
 	if (opt_divertto != NULL &&
 	    strcmp(opt_divertto, contest->useinstead->name) != 0)
 		ohshit(_("mismatch on divert-to\n"
-		         "  when removing `%s'\n"
-		         "  found `%s'"),
+		         "  when removing '%s'\n"
+		         "  found '%s'"),
 		       diversion_current(filename),
 		       diversion_describe(contest));
 
 	if (!opt_pkgname_match_any && pkgset != contest->pkgset)
 		ohshit(_("mismatch on package\n"
-		         "  when removing `%s'\n"
-		         "  found `%s'"),
+		         "  when removing '%s'\n"
+		         "  found '%s'"),
 		       diversion_current(filename),
 		       diversion_describe(contest));
 
