@@ -3,7 +3,7 @@
  * enquiry.c - status enquiry and listing options
  *
  * Copyright © 1995,1996 Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2006,2008-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006, 2008-2015 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -21,8 +21,6 @@
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-/* FIXME: per-package audit. */
-
 #include <config.h>
 #include <compat.h>
 
@@ -40,6 +38,7 @@
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
 #include <dpkg/arch.h>
+#include <dpkg/pkg-array.h>
 #include <dpkg/pkg-show.h>
 #include <dpkg/string.h>
 #include <dpkg/options.h>
@@ -48,8 +47,8 @@
 #include "infodb.h"
 #include "main.h"
 
-struct badstatinfo {
-  bool (*yesno)(struct pkginfo *, const struct badstatinfo *bsi);
+struct audit_problem {
+  bool (*check)(struct pkginfo *, const struct audit_problem *problem);
   union {
     int number;
     const char *string;
@@ -58,107 +57,107 @@ struct badstatinfo {
 };
 
 static bool
-bsyn_reinstreq(struct pkginfo *pkg, const struct badstatinfo *bsi)
+audit_reinstreq(struct pkginfo *pkg, const struct audit_problem *problem)
 {
-  return pkg->eflag & eflag_reinstreq;
+  return pkg->eflag & PKG_EFLAG_REINSTREQ;
 }
 
 static bool
-bsyn_status(struct pkginfo *pkg, const struct badstatinfo *bsi)
+audit_status(struct pkginfo *pkg, const struct audit_problem *problem)
 {
-  if (pkg->eflag & eflag_reinstreq)
+  if (pkg->eflag & PKG_EFLAG_REINSTREQ)
     return false;
-  return (int)pkg->status == bsi->value.number;
+  return (int)pkg->status == problem->value.number;
 }
 
 static bool
-bsyn_infofile(struct pkginfo *pkg, const struct badstatinfo *bsi)
+audit_infofile(struct pkginfo *pkg, const struct audit_problem *problem)
 {
-  if (pkg->status < stat_halfinstalled)
+  if (pkg->status < PKG_STAT_HALFINSTALLED)
     return false;
-  return !pkg_infodb_has_file(pkg, &pkg->installed, bsi->value.string);
+  return !pkg_infodb_has_file(pkg, &pkg->installed, problem->value.string);
 }
 
 static bool
-bsyn_arch(struct pkginfo *pkg, const struct badstatinfo *bsi)
+audit_arch(struct pkginfo *pkg, const struct audit_problem *problem)
 {
-  if (pkg->status < stat_halfinstalled)
+  if (pkg->status < PKG_STAT_HALFINSTALLED)
     return false;
-  return pkg->installed.arch->type == (enum dpkg_arch_type)bsi->value.number;
+  return pkg->installed.arch->type == (enum dpkg_arch_type)problem->value.number;
 }
 
-static const struct badstatinfo badstatinfos[]= {
+static const struct audit_problem audit_problems[] = {
   {
-    .yesno = bsyn_reinstreq,
+    .check = audit_reinstreq,
     .value.number = 0,
     .explanation = N_(
     "The following packages are in a mess due to serious problems during\n"
     "installation.  They must be reinstalled for them (and any packages\n"
     "that depend on them) to function properly:\n")
   }, {
-    .yesno = bsyn_status,
-    .value.number = stat_unpacked,
+    .check = audit_status,
+    .value.number = PKG_STAT_UNPACKED,
     .explanation = N_(
     "The following packages have been unpacked but not yet configured.\n"
     "They must be configured using dpkg --configure or the configure\n"
     "menu option in dselect for them to work:\n")
   }, {
-    .yesno = bsyn_status,
-    .value.number = stat_halfconfigured,
+    .check = audit_status,
+    .value.number = PKG_STAT_HALFCONFIGURED,
     .explanation = N_(
     "The following packages are only half configured, probably due to problems\n"
     "configuring them the first time.  The configuration should be retried using\n"
     "dpkg --configure <package> or the configure menu option in dselect:\n")
   }, {
-    .yesno = bsyn_status,
-    .value.number = stat_halfinstalled,
+    .check = audit_status,
+    .value.number = PKG_STAT_HALFINSTALLED,
     .explanation = N_(
     "The following packages are only half installed, due to problems during\n"
     "installation.  The installation can probably be completed by retrying it;\n"
     "the packages can be removed using dselect or dpkg --remove:\n")
   }, {
-    .yesno = bsyn_status,
-    .value.number = stat_triggersawaited,
+    .check = audit_status,
+    .value.number = PKG_STAT_TRIGGERSAWAITED,
     .explanation = N_(
     "The following packages are awaiting processing of triggers that they\n"
     "have activated in other packages.  This processing can be requested using\n"
     "dselect or dpkg --configure --pending (or dpkg --triggers-only):\n")
   }, {
-    .yesno = bsyn_status,
-    .value.number = stat_triggerspending,
+    .check = audit_status,
+    .value.number = PKG_STAT_TRIGGERSPENDING,
     .explanation = N_(
     "The following packages have been triggered, but the trigger processing\n"
     "has not yet been done.  Trigger processing can be requested using\n"
     "dselect or dpkg --configure --pending (or dpkg --triggers-only):\n")
   }, {
-    .yesno = bsyn_infofile,
+    .check = audit_infofile,
     .value.string = LISTFILE,
     .explanation = N_(
     "The following packages are missing the list control file in the\n"
     "database, they need to be reinstalled:\n")
   }, {
-    .yesno = bsyn_infofile,
+    .check = audit_infofile,
     .value.string = HASHFILE,
     .explanation = N_(
     "The following packages are missing the md5sums control file in the\n"
     "database, they need to be reinstalled:\n")
   }, {
-    .yesno = bsyn_arch,
-    .value.number = arch_none,
+    .check = audit_arch,
+    .value.number = DPKG_ARCH_EMPTY,
     .explanation = N_("The following packages do not have an architecture:\n")
   }, {
-    .yesno = bsyn_arch,
-    .value.number = arch_illegal,
+    .check = audit_arch,
+    .value.number = DPKG_ARCH_ILLEGAL,
     .explanation = N_("The following packages have an illegal architecture:\n")
   }, {
-    .yesno = bsyn_arch,
-    .value.number = arch_unknown,
+    .check = audit_arch,
+    .value.number = DPKG_ARCH_UNKNOWN,
     .explanation = N_(
     "The following packages have an unknown foreign architecture, which will\n"
     "cause dependency issues on front-ends. This can be fixed by registering\n"
     "the foreign architecture with dpkg --add-architecture:\n")
   }, {
-    .yesno = NULL
+    .check = NULL
   }
 };
 
@@ -176,25 +175,43 @@ static void describebriefly(struct pkginfo *pkg) {
   printf(" %-20s %.*s\n", pkg_name(pkg, pnaw_nonambig), l, pdesc);
 }
 
+static struct pkginfo *
+pkg_array_mapper(const char *name)
+{
+  struct pkginfo *pkg;
+
+  pkg = dpkg_options_parse_pkgname(cipaction, name);
+  if (pkg->status == PKG_STAT_NOTINSTALLED)
+    notice(_("package '%s' is not installed"), pkg_name(pkg, pnaw_nonambig));
+
+  return pkg;
+}
+
 int
 audit(const char *const *argv)
 {
-  const struct badstatinfo *bsi;
+  const struct audit_problem *problem;
+  struct pkg_array array;
   bool head_running = false;
-
-  if (*argv)
-    badusage(_("--%s takes no arguments"), cipaction->olong);
+  int i;
 
   modstatdb_open(msdbrw_readonly);
 
-  for (bsi= badstatinfos; bsi->yesno; bsi++) {
-    struct pkgiterator *it;
-    struct pkginfo *pkg;
+  if (!*argv)
+    pkg_array_init_from_db(&array);
+  else
+    pkg_array_init_from_names(&array, pkg_array_mapper, (const char **)argv);
+
+  pkg_array_sort(&array, pkg_sorter_by_nonambig_name_arch);
+
+  for (problem = audit_problems; problem->check; problem++) {
     bool head = false;
 
-    it = pkg_db_iter_new();
-    while ((pkg = pkg_db_iter_next_pkg(it))) {
-      if (!bsi->yesno(pkg,bsi)) continue;
+    for (i = 0; i < array.n_pkgs; i++) {
+      struct pkginfo *pkg = array.pkgs[i];
+
+      if (!problem->check(pkg, problem))
+        continue;
       if (!head_running) {
         if (modstatdb_is_locked())
           puts(_(
@@ -203,15 +220,17 @@ audit(const char *const *argv)
         head_running = true;
       }
       if (!head) {
-        fputs(gettext(bsi->explanation),stdout);
+        fputs(gettext(problem->explanation), stdout);
         head = true;
       }
       describebriefly(pkg);
     }
-    pkg_db_iter_free(it);
+
     if (head) putchar('\n');
   }
 
+  pkg_array_destroy(&array);
+
   m_output(stdout, _("<standard output>"));
 
   return 0;
@@ -226,15 +245,19 @@ struct sectionentry {
 static bool
 yettobeunpacked(struct pkginfo *pkg, const char **thissect)
 {
-  if (pkg->want != want_install)
+  if (pkg->want != PKG_WANT_INSTALL)
     return false;
 
   switch (pkg->status) {
-  case stat_unpacked: case stat_installed: case stat_halfconfigured:
-  case stat_triggerspending:
-  case stat_triggersawaited:
+  case PKG_STAT_UNPACKED:
+  case PKG_STAT_INSTALLED:
+  case PKG_STAT_HALFCONFIGURED:
+  case PKG_STAT_TRIGGERSPENDING:
+  case PKG_STAT_TRIGGERSAWAITED:
     return false;
-  case stat_notinstalled: case stat_halfinstalled: case stat_configfiles:
+  case PKG_STAT_NOTINSTALLED:
+  case PKG_STAT_HALFINSTALLED:
+  case PKG_STAT_CONFIGFILES:
     if (thissect)
       *thissect = str_is_set(pkg->section) ? pkg->section :
                                              C_("section", "<unknown>");
@@ -348,12 +371,14 @@ assert_version_support(const char *const *argv,
 
   pkg = pkg_db_find_singleton("dpkg");
   switch (pkg->status) {
-  case stat_installed:
-  case stat_triggerspending:
+  case PKG_STAT_INSTALLED:
+  case PKG_STAT_TRIGGERSPENDING:
     return 0;
-  case stat_unpacked: case stat_halfconfigured: case stat_halfinstalled:
-  case stat_triggersawaited:
-    if (dpkg_version_relate(&pkg->configversion, dpkg_relation_ge, version))
+  case PKG_STAT_UNPACKED:
+  case PKG_STAT_HALFCONFIGURED:
+  case PKG_STAT_HALFINSTALLED:
+  case PKG_STAT_TRIGGERSAWAITED:
+    if (dpkg_version_relate(&pkg->configversion, DPKG_RELATION_GE, version))
       return 0;
     printf(_("Version of dpkg with working %s support not yet configured.\n"
              " Please use 'dpkg --configure dpkg', and then try again.\n"),
@@ -407,6 +432,14 @@ assertmultiarch(const char *const *argv)
   return assert_version_support(argv, &version, _("multi-arch"));
 }
 
+int
+assertverprovides(const char *const *argv)
+{
+  struct dpkg_version version = { 0, "1.17.11", NULL };
+
+  return assert_version_support(argv, &version, _("versioned Provides"));
+}
+
 /**
  * Print a single package which:
  *  (a) is the target of one or more relevant predependencies.
@@ -441,12 +474,12 @@ predeppackage(const char *const *argv)
   it = pkg_db_iter_new();
   while (!dep && (pkg = pkg_db_iter_next_pkg(it))) {
     /* Ignore packages user doesn't want. */
-    if (pkg->want != want_install)
+    if (pkg->want != PKG_WANT_INSTALL)
       continue;
     /* Ignore packages not available. */
     if (!pkg->files)
       continue;
-    pkg->clientdata->istobe= itb_preinstall;
+    pkg->clientdata->istobe = PKG_ISTOBE_PREINSTALL;
     for (dep= pkg->available.depends; dep; dep= dep->next) {
       if (dep->type != dep_predepends) continue;
       if (depisok(dep, &vb, NULL, NULL, true))
@@ -454,7 +487,7 @@ predeppackage(const char *const *argv)
       /* This will leave dep non-NULL, and so exit the loop. */
       break;
     }
-    pkg->clientdata->istobe= itb_normal;
+    pkg->clientdata->istobe = PKG_ISTOBE_NORMAL;
     /* If dep is NULL we go and get the next package. */
   }
   pkg_db_iter_free(it);
@@ -463,7 +496,7 @@ predeppackage(const char *const *argv)
     return 1; /* Not found. */
   assert(pkg);
   startpkg= pkg;
-  pkg->clientdata->istobe= itb_preinstall;
+  pkg->clientdata->istobe = PKG_ISTOBE_PREINSTALL;
 
   /* OK, we have found an unsatisfied predependency.
    * Now go and find the first thing we need to install, as a first step
@@ -477,22 +510,23 @@ predeppackage(const char *const *argv)
 
       possi_iter = deppossi_pkg_iter_new(possi, wpb_available);
       while (!pkg && (trypkg = deppossi_pkg_iter_next(possi_iter))) {
-        if (trypkg->files && trypkg->clientdata->istobe == itb_normal &&
+        if (trypkg->files &&
+            trypkg->clientdata->istobe == PKG_ISTOBE_NORMAL &&
             versionsatisfied(&trypkg->available, possi)) {
           pkg = trypkg;
           break;
         }
-        if (possi->verrel != dpkg_relation_none)
-          continue;
         for (provider = possi->ed->depended.available;
              !pkg && provider;
              provider = provider->next) {
           if (provider->up->type != dep_provides)
             continue;
+          if (!pkg_virtual_deppossi_satisfied(possi, provider))
+            continue;
           trypkg = provider->up->up;
           if (!trypkg->files)
             continue;
-          if (trypkg->clientdata->istobe == itb_normal) {
+          if (trypkg->clientdata->istobe == PKG_ISTOBE_NORMAL) {
             pkg = trypkg;
             break;
           }
@@ -509,7 +543,7 @@ predeppackage(const char *const *argv)
              pkgbin_name(dep->up, &dep->up->available, pnaw_nonambig),
              pkgbin_name(startpkg, &startpkg->available, pnaw_nonambig));
     }
-    pkg->clientdata->istobe= itb_preinstall;
+    pkg->clientdata->istobe = PKG_ISTOBE_PREINSTALL;
     for (dep= pkg->available.depends; dep; dep= dep->next) {
       if (dep->type != dep_predepends) continue;
       if (depisok(dep, &vb, NULL, NULL, true))
@@ -533,7 +567,7 @@ printarch(const char *const *argv)
   if (*argv)
     badusage(_("--%s takes no arguments"), cipaction->olong);
 
-  printf("%s\n", dpkg_arch_get(arch_native)->name);
+  printf("%s\n", dpkg_arch_get(DPKG_ARCH_NATIVE)->name);
 
   m_output(stdout, _("<standard output>"));
 
@@ -559,7 +593,7 @@ print_foreign_arches(const char *const *argv)
   dpkg_arch_load_list();
 
   for (arch = dpkg_arch_get_list(); arch; arch = arch->next) {
-    if (arch->type != arch_foreign)
+    if (arch->type != DPKG_ARCH_FOREIGN)
       continue;
 
     printf("%s\n", arch->name);
@@ -578,6 +612,7 @@ cmpversions(const char *const *argv)
     /* These values are exit status codes, so 0 = true, 1 = false. */
     int if_lesser, if_equal, if_greater;
     int if_none_a, if_none_both, if_none_b;
+    bool obsolete;
   };
 
   static const struct relationinfo relationinfos[]= {
@@ -596,11 +631,11 @@ cmpversions(const char *const *argv)
     { "gt-nl",     1,1,0, 0,1,1  },
 
     /* For compatibility with dpkg control file syntax. */
-    { "<",         0,0,1, 0,0,1  },
+    { "<",         0,0,1, 0,0,1, .obsolete = true },
     { "<=",        0,0,1, 0,0,1  },
     { "<<",        0,1,1, 0,1,1  },
     { "=",         1,0,1, 1,0,1  },
-    { ">",         1,0,0, 1,0,0  },
+    { ">",         1,0,0, 1,0,0, .obsolete = true },
     { ">=",        1,0,0, 1,0,0  },
     { ">>",        1,1,0, 1,1,0  },
     { NULL                       }
@@ -609,7 +644,7 @@ cmpversions(const char *const *argv)
   const struct relationinfo *rip;
   struct dpkg_version a, b;
   struct dpkg_error err;
-  int r;
+  int rc;
 
   if (!argv[0] || !argv[1] || !argv[2] || argv[3])
     badusage(_("--compare-versions takes three arguments:"
@@ -619,12 +654,13 @@ cmpversions(const char *const *argv)
 
   if (!rip->string) badusage(_("--compare-versions bad relation"));
 
+  if (rip->obsolete)
+    warning(_("--%s used with obsolete relation operator '%s'"),
+            cipaction->olong, rip->string);
+
   if (*argv[0] && strcmp(argv[0],"<unknown>")) {
     if (parseversion(&a, argv[0], &err) < 0) {
-      if (err.type == DPKG_MSG_WARN)
-        warning(_("version '%s' has bad syntax: %s"), argv[0], err.str);
-      else
-        ohshit(_("version '%s' has bad syntax: %s"), argv[0], err.str);
+      dpkg_error_print(&err, _("version '%s' has bad syntax"), argv[0]);
       dpkg_error_destroy(&err);
     }
   } else {
@@ -632,10 +668,7 @@ cmpversions(const char *const *argv)
   }
   if (*argv[2] && strcmp(argv[2],"<unknown>")) {
     if (parseversion(&b, argv[2], &err) < 0) {
-      if (err.type == DPKG_MSG_WARN)
-        warning(_("version '%s' has bad syntax: %s"), argv[2], err.str);
-      else
-        ohshit(_("version '%s' has bad syntax: %s"), argv[2], err.str);
+      dpkg_error_print(&err, _("version '%s' has bad syntax"), argv[2]);
       dpkg_error_destroy(&err);
     }
   } else {
@@ -649,14 +682,14 @@ cmpversions(const char *const *argv)
   } else if (!dpkg_version_is_informative(&b)) {
     return rip->if_none_b;
   }
-  r = dpkg_version_compare(&a, &b);
-  debug(dbg_general,"cmpversions a=`%s' b=`%s' r=%d",
+  rc = dpkg_version_compare(&a, &b);
+  debug(dbg_general, "cmpversions a='%s' b='%s' r=%d",
         versiondescribe(&a,vdew_always),
         versiondescribe(&b,vdew_always),
-        r);
-  if (r > 0)
+        rc);
+  if (rc > 0)
     return rip->if_greater;
-  else if (r < 0)
+  else if (rc < 0)
     return rip->if_lesser;
   else
     return rip->if_equal;
